package containers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import fuzzySet.FuzzyNumber;
import fuzzySet.MembershipFunction;
import fuzzySet.TrapezoidalFunction;


public class TrapezoidalFuzzyNumberContainer implements FuzzyNumberContainer {

	private List<FuzzyNumber> fuzzyNumbers;
	private MembershipFunction function;

	public TrapezoidalFuzzyNumberContainer() {
		fuzzyNumbers = new ArrayList<FuzzyNumber>();
		function = new TrapezoidalFunction();
	}

	public void addFuzzyNumber(FuzzyNumber fn) {
		fuzzyNumbers.add(fn);
	}

	public void clearFuzzyNumbersContainer() {
		fuzzyNumbers.clear();
	}

	public int getSize() {
		return fuzzyNumbers.size();
	}

	// returns the fuzzy number with the highest degree for a given value
	public FuzzyNumber getFuzzyNumberByValue(double value) {

		Iterator<FuzzyNumber> itr = fuzzyNumbers.iterator();
		double maxDegree = 0;
		FuzzyNumber primaryNumber = null;

		while (itr.hasNext()) {
			FuzzyNumber fn = itr.next();
			double degree = function.getMembershipDegree(fn, value);

			if (degree == 0.5)
				System.out.println("Degree 0.5!!!!!!!!");
			if (degree > maxDegree) {
				maxDegree = degree;
				primaryNumber = fn;
			}

		}
		return primaryNumber;
	}

	public FuzzyNumber getFuzzyNumberByIndex(int index) {
		return fuzzyNumbers.get(index);
	}

	public List<FuzzyNumber> getFuzzyNumberContainer() {
		return fuzzyNumbers;
	}

	@Override
	public Iterator<FuzzyNumber> getFuzzyNumberContainerIterator() {
		// TODO Auto-generated method stub
		return fuzzyNumbers.iterator();
	}
}
