package particleSwarm;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

import valueObjects.ActualValue;

import fuzzySet.FuzzySet;
import fuzzySet.FuzzySetsRelationship;
import fuzzySet.TrapezoidalFuzzySet;

public class Particle {

	private double positions[];
	// private double fuzzyValues[];
	private List<ActualValue> actualValues;
	private double localBestPos[];
	private double velocities[];
	private ActualValue actualValue;
	private FuzzySetsRelationship relationship;

	public Particle(List<ActualValue> actualValues,
			FuzzySetsRelationship relationship, ActualValue actualValue) {
		this.actualValues = actualValues;
		this.relationship = relationship;
		this.actualValue = actualValue;
	}

	public void setVelocities(double velocities[]) {
		this.velocities = velocities;
	}

	public double[] getVelocities() {
		return velocities;
	}

	public void setActualValue(ActualValue actualValue) {
		this.actualValue = actualValue;
	}

	public ActualValue getActualValue() {
		return actualValue;
	}

	public double[] getLocalBestPositions() {
		return localBestPos;
	}

	public void setLocalBestPositions(double localBestPos[]) {
		this.localBestPos = localBestPos;
	}

	public double[] getPositions() {
		return positions;
	}

	public void setPositions(double positions[]) {
		this.positions = positions;
	}

	public double getSquaredError() {

		double sum = getSum();
		double error = sum - actualValue.getActualValue();
		double squaredError = Math.pow(error, 2);
		double roundedSquaredError = new BigDecimal(squaredError).setScale(3,
				RoundingMode.HALF_UP).doubleValue();
		return roundedSquaredError;
	}

	public double getSum() {

		double sum = 0;

		// List<FuzzySet> fuzzySets = relationship.getLeftHandSidesContainer();

		for (int i = 0; i < actualValues.size(); i++) {

			double value = actualValues.get(i).getActualValue();

			// double centroid = set.getCentroid();
			double tempNumber = value * positions[i];
			sum += tempNumber;
		}
		double roundedSum = new BigDecimal(sum).setScale(3,
				RoundingMode.HALF_UP).doubleValue();
		return roundedSum;
	}

	public String toString() {

		String tempString = "Fuzzy Values : ";

		List<FuzzySet> container = relationship.getLeftHandSidesContainer();
		int length = container.size();

		for (int i = 0; i < length; i++) {

			TrapezoidalFuzzySet set = (TrapezoidalFuzzySet) container.get(i);
			double centroid = new BigDecimal(set.getCentroid()).setScale(0,
					RoundingMode.HALF_UP).doubleValue();

			if (i == length - 1)
				tempString += centroid + "\n";
			else
				tempString += centroid + ", ";
		}

		tempString += "ActualValues : ";

		for (int i = 0; i < actualValues.size(); i++) {

			if (i == length - 1)
				tempString += actualValues.get(i).getActualValue() + "\n";
			else
				tempString += actualValues.get(i).getActualValue() + ", ";
		}

		tempString += "Weights: ";

		for (int i = 0; i < positions.length; i++) {

			if (i == positions.length - 1)
				tempString += positions[i] + "\n";
			else
				tempString += positions[i] + ", ";
		}

		tempString += "Personal best positions : ";

		for (int i = 0; i < localBestPos.length; i++) {

			if (i == localBestPos.length - 1)
				tempString += localBestPos[i] + "\n";
			else
				tempString += localBestPos[i] + ", ";
		}

		tempString += "Velocities: ";

		for (int i = 0; i < velocities.length; i++) {

			if (i == velocities.length - 1)
				tempString += velocities[i] + "\n";
			else
				tempString += velocities[i] + ", ";
		}
		return tempString;

	}
}
