package particleSwarm;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.lang.Math;

import valueObjects.ActualValue;

import containers.ParticleContainer;
import fuzzySet.FuzzySetsRelationship;

public class ParticleSwarmer {

	private ActualValue actualValue;
	private FuzzySetsRelationship relationship;
	private List<ActualValue> actualValues;

	private static final double MAX_VELOCITY = 1d / 100;
	private static final double MIN_VELOCITY = -1d / 100;
	private static final int MAX_ITERATIONS = 500;
	private static final double MIN_SQUARED_ERROR = 3;
	private static final int C1 = 2;
	private static final int C2 = 2;
	private static final double FUNCTION = 1.4;
	private static final int NUMBER_OF_PARTICLES = 5;
	private static final int NUMBER_OF_DECIMALS = 4;
	private int order;
	// private static final int NUMBER_OF_WEIGHTS = 3;

	private ParticleContainer particleContainer = new ParticleContainer();
	private Particle globalBestParticle = null;

	public ParticleSwarmer(List<ActualValue> actualValues,
			FuzzySetsRelationship relationship, int order,
			ActualValue actualValue) {
		this.relationship = relationship;
		this.actualValues = actualValues;
		this.actualValue = actualValue;
		this.order = order;
		initializeParticles();
		particleSwarm();
	}

	public ParticleContainer getParticles() {
		return particleContainer;
	}

	public Particle getGlobalBestParticle() {
		return globalBestParticle;
	}

	private void setGlobalBestParticle(Particle globalBestParticle) {
		this.globalBestParticle = globalBestParticle;
	}

	private void initializeParticles() {

		for (int i = 0; i < ParticleSwarmer.NUMBER_OF_PARTICLES; i++) {

			Particle p = new Particle(actualValues, relationship, actualValue);
			double positions[] = new double[order];
			double localBestPos[] = new double[order];
			double velocities[] = new double[order];

			double order = ((double) relationship.getSize());
			// BigDecimal randWeight = new BigDecimal(1d / order - random);
			// double roundedRandWeight = randWeight.setScale(
			// ParticleSwarmer.NUMBER_OF_DECIMALS, RoundingMode.HALF_UP)
			// .doubleValue();
			double roundedRandWeight = 1 / order;

			// if (roundedRandWeight < 0)
			// roundedRandWeight = 0d;
			// if (roundedRandWeight > 1)
			// roundedRandWeight = 1d;

			for (int j = 0; j < order; j++) {

				positions[j] = roundedRandWeight;
				localBestPos[j] = roundedRandWeight;
				// double random = (Math.random() / 10);

				// randWeight = new BigDecimal(roundedRandWeight + random);
				double random = (1 / order) * (1 / order);
				roundedRandWeight = new BigDecimal(roundedRandWeight + random)
						.setScale(ParticleSwarmer.NUMBER_OF_DECIMALS,
								RoundingMode.HALF_UP).doubleValue();
				// random = (Math.random() / 10);

				double randomVelocity = new BigDecimal(Math.random()
						* ParticleSwarmer.MAX_VELOCITY).setScale(
						ParticleSwarmer.NUMBER_OF_DECIMALS,
						RoundingMode.HALF_UP).doubleValue();
				velocities[j] = randomVelocity;
			}
			p.setPositions(positions);
			p.setLocalBestPositions(localBestPos);
			p.setVelocities(velocities);
			particleContainer.addParticle(p);
			// System.out.println("initial particle " + p);
			// System.out.println("SE " + p.getSquaredError());
		}
		this.setGlobalBestParticle(particleContainer.getGlobalBestPosition());
		// globalBestParticle = ;
	}

	private void update() {
		updateVelocities();
		updatePositions();
	}

	private void updateVelocities() {

		double globalBestPos[] = globalBestParticle.getPositions();

		for (int i = 0; i < particleContainer.getContainerSize(); i++) {

			Particle p = particleContainer.getParticle(i);
			double velocities[] = p.getVelocities();
			double positions[] = p.getPositions();
			double localBestPos[] = p.getLocalBestPositions();

			for (int j = 0; j < velocities.length; j++) {

				double randVal1 = new BigDecimal(Math.random()).setScale(
						ParticleSwarmer.NUMBER_OF_DECIMALS,
						RoundingMode.HALF_UP).doubleValue();
				double randVal2 = new BigDecimal(Math.random()).setScale(
						ParticleSwarmer.NUMBER_OF_DECIMALS,
						RoundingMode.HALF_UP).doubleValue();

				double temp = (ParticleSwarmer.FUNCTION * velocities[j])
						+ (ParticleSwarmer.C1 * randVal1)
						* (localBestPos[j] - positions[j])
						+ (ParticleSwarmer.C2 * randVal2)
						* (globalBestPos[j] - positions[j]);

				double tempVelocity = new BigDecimal(temp).setScale(
						ParticleSwarmer.NUMBER_OF_DECIMALS,
						RoundingMode.HALF_UP).doubleValue();
				// System.out.println("------- tempVel -------" + tempVelocity);

				// System.out.println("tempVel " + tempVelocity);

				if (tempVelocity >= ParticleSwarmer.MIN_VELOCITY
						&& tempVelocity <= ParticleSwarmer.MAX_VELOCITY) {
					velocities[j] = tempVelocity;
					// System.out.println("if 1 velocity " + velocities[j]);
				} else {
					if (tempVelocity < ParticleSwarmer.MIN_VELOCITY) {
						velocities[j] = ParticleSwarmer.MIN_VELOCITY;
						// System.out.println("if 2 velocity = " +
						// velocities[j]);
					} else {
						velocities[j] = ParticleSwarmer.MAX_VELOCITY;
						// System.out.println("if 3 velocity = " +
						// velocities[j]);
					}
				}
				// System.out.println("Updated Velocity " + i + "
				// "+velocities[j]);
				// System.out.println("Rand 1 " + randVal1);
				// System.out.println("Rand 2 " + randVal2);
			}

		}
	}

	private void updatePositions() {

		for (int i = 0; i < particleContainer.getContainerSize(); i++) {

			Particle p = particleContainer.getParticle(i);
			double positions[] = p.getPositions();
			double bestPos[] = p.getLocalBestPositions();
			double velocities[] = p.getVelocities();
			double squaredError1 = p.getSquaredError();

			for (int j = 0; j < positions.length; j++) {
				double temp = positions[j] + velocities[j];
				double tempPos = new BigDecimal(temp).setScale(
						ParticleSwarmer.NUMBER_OF_DECIMALS,
						RoundingMode.HALF_UP).doubleValue();
				positions[j] = tempPos;
			}

			// update best local positions
			double squaredError2 = p.getSquaredError();
			if (squaredError2 < squaredError1) {
				for (int k = 0; k < bestPos.length; k++) {
					bestPos[k] = positions[k];
				}
			}
		}
		// update global best pos
		Particle tempGlobalBest = particleContainer.getGlobalBestPosition();
		if (tempGlobalBest.getSquaredError() < globalBestParticle
				.getSquaredError())
			globalBestParticle = tempGlobalBest;
	}

	private Particle particleSwarm() {

		int counter = 0;
		boolean finish = true;
		// System.out.println("---------------- Before while -------------- ");
		while (finish && counter <= ParticleSwarmer.MAX_ITERATIONS) {
			update();
			double squaredError = globalBestParticle.getSquaredError();

			// if()

			if (squaredError < ParticleSwarmer.MIN_SQUARED_ERROR) {
				// System.out
				// .println(" ----- squaredError < minSquaredError ---- ");
				finish = false;
			}
			counter++;
		}
		// System.out.println("---------------- After while -------------- ");
		return globalBestParticle;
	}
}
