package test;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import particleSwarm.Particle;
import particleSwarm.Particle2;
import particleSwarm.ParticleSwarmer;
import performanceMeasures.ForecastError;

import containers.ActualValueContainer;
import containers.AlabamaEnrollmentsContainer;
import containers.ForecastValueContainer;
import containers.RelationshipContainer;

import valueObjects.ActualValue;
import valueObjects.ForecastValue;
import fuzzySet.FuzzySet;
import fuzzySet.FuzzySetsRelationship;
import fuzzySet.TrapezoidalFuzzifier;

public class ParticleSwarmTester {

	private static ActualValueContainer actualValues = new AlabamaEnrollmentsContainer()
			.getActualEnrollmentsContainer();
	private TrapezoidalFuzzifier fuzzifier = new TrapezoidalFuzzifier(
			actualValues);
	private RelationshipContainer relationships = new RelationshipContainer();
	ForecastValueContainer forecastValues = new ForecastValueContainer();
	private static final int order = 3;
	private int year = 1971 + order;

	private static List<List> tempList = new ArrayList<List>();

	public ParticleSwarmTester() {
		initRelationships();
		initActualValues();

	}

	public ForecastValueContainer getForecasts() {

		Iterator<FuzzySetsRelationship> itr = relationships
				.getRelationshipsIterator();
		Iterator<List> itr3 = tempList.iterator();
		int counter = order;

		while (itr.hasNext()) {

			FuzzySetsRelationship rel = itr.next();
			List l = itr3.next();
			ParticleSwarmer psw = new ParticleSwarmer(l, rel, order,
					actualValues.getActualValue(counter));

			if (checkParticle(psw.getGlobalBestParticle())) {
				// System.out.println("------------ Error -----------------");
				while (checkParticle(psw.getGlobalBestParticle())) {
					psw = new ParticleSwarmer(l, rel, order, actualValues
							.getActualValue(counter));
				}
			}
			Particle bestParticle = psw.getGlobalBestParticle();

			// System.out.println("Global Best " + psw.getGlobalBestParticle());
//			Iterator<Particle> itr2 = psw.getParticles()
//					.getParticleContainerIterator();
			// System.out.println("Final particles ");
			// while (itr2.hasNext()) {
			// System.out.println(itr2.next());
			//
			//			}
			double value = new BigDecimal(bestParticle.getSum()).setScale(0,
					RoundingMode.HALF_UP).doubleValue();
			ForecastValue fv = new ForecastValue("" + year++, value);
			rel.setForecastValue(fv);
			forecastValues.addForecastValue(fv);
			counter++;
			System.out.println(rel);
		}
		return forecastValues;
	}

	private boolean checkParticle(Particle p) {

		double positions[] = p.getPositions();

		for (int i = 0; i < positions.length; i++) {
			if (positions[i] < 0 || positions[i] > 1) {
				return true;
			}
		}
		return false;
	}

	private void initRelationships() {

		int numberOfRelationships = fuzzifier.getNumberOfSets() - order + 1;
		int index = 0;
		int stopIndex = order;

		for (int j = 0; j < numberOfRelationships - 1; j++) {

			FuzzySetsRelationship newRelationship = new FuzzySetsRelationship();

			while (index < stopIndex) {
				FuzzySet set = fuzzifier.getFuzzySet(index);
				newRelationship.addFuzzySet(set);
				index++;
			}
			relationships.addRelationship(newRelationship);
			index = index - order + 1;
			stopIndex++;
		}
	}

	private void initActualValues() {

		int numberOfRelationships = fuzzifier.getNumberOfSets() - order + 1;
		int index = 0;
		int stopIndex = order;

		for (int j = 0; j < numberOfRelationships - 1; j++) {

			List<ActualValue> temp = new ArrayList<ActualValue>();

			while (index < stopIndex) {
				ActualValue val = actualValues.getActualValue(index);
				temp.add(val);
				index++;
			}
			tempList.add(temp);
			index = index - order + 1;
			stopIndex++;
		}

	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

		// ----------------------------------------------------------------
		ParticleSwarmTester partTest = new ParticleSwarmTester();
		ForecastValueContainer forecastValues = partTest.getForecasts();
		// ---------------------------------------------------------------
		Iterator itr = forecastValues.getForecastValuesIterator();

		while (itr.hasNext()) {

			System.out.println(itr.next());

		}
		// -------------------------------------------------------------------
		ForecastError error = new ForecastError(forecastValues, actualValues,
				order);
		System.out.println("MSE = " + error.getMSE());
		System.out.println("MAPE = " + error.getMAPE());

		// -------------------------------------------------------------------
	}
}
